@@ -39,11 +39,17 @@ def Simulator(q_in, q_out, q_intermediate):
         q_out.put(msg)
 
 class Simulate:
-    def __init__(self, army_a: Optional[Army], army_b: Optional[Army]):
-        self.N = 2000
+    def __init__(self, 
+                    army_a: Optional[Army], 
+                    army_b: Optional[Army], 
+                    config = None,
+                    combat_system: Optional[CombatSystem] = None):
+        self.N = 5000
         self.army_a = army_a
         self.army_b = army_b
         self.battle_type = 'land'
+        self.combat_system = combat_system
+        self.config = config
 
         self.statistics = []
         self.survivors = {'A':{'land':[],
@@ -84,7 +90,54 @@ class Simulate:
         if self.army_a is not None and self.army_b is not None:
             if np.sum(self.army_a.units['sea']) + np.sum(self.army_b.units['sea']) > 0:
                 self.battle_type = 'sea'
-        
+
+
+    def run_cb(self, app):
+        self.update_battle_type()
+        troops_a, troops_b = 0, 0
+        for T in ['sea', 'land', 'air']:
+            troops_a += self.army_a.units[T].sum() 
+            troops_b += self.army_b.units[T].sum() 
+
+        if troops_a == 0 or troops_b == 0:
+            return
+
+        if self.combat_system is CombatSystem.WarRoomV2 and self.config is None:
+            logging.warning("NO CONFIG provided falling back to default config!!!!")
+            self.config = wr20_vaniilla_options
+        elif self.config is None:
+            raise RuntimeError("No Config for combat system found")
+
+
+        for n in tqdm(range(self.N)):
+            self.cur_n = n
+            battle = Battle(copy.copy(self.army_a), 
+                            copy.copy(self.army_b),
+                            self.config)
+            status = battle.run(combat_system=self.combat_system)
+            
+            abrt = self.running_stats(status)
+            self.statistics.append(status)
+
+            for side in ['A', 'B']:
+                for type in ['land', 'air', 'sea']:
+                    self.survivors[side][type].append(battle.army[side].units[type])
+
+            app.spinner.value = int(n / self.N * 100 )
+
+            if abrt or n+1 == self.N:
+                app.spinner.value = 100
+                app.spinner.stop()
+
+                self.eval_statistics()
+                app.results.text = self.get_report()
+                return
+
+            if n % 50 == 0 and n != self.N:
+                app.results.text = self.intermediate_statistics()
+                yield 0.01
+
+
 
     def run_all(self, combat_system: CombatSystem, config=None,  armyA: Optional[Army]=None, armyB: Optional[Army]=None, q_intermediate=None) -> StringIO:
         if armyA is not None:
@@ -129,19 +182,21 @@ class Simulate:
 
 
 
+    def run_async(self, combat_system: CombatSystem, config=None, armyA: Optional[Army]=None, armyB: Optional[Army]=None) -> int:
+        if armyA is not None:
+            self.army_a = armyA
+        if armyB is not None:
+            self.army_b = armyB
 
+        self.update_battle_type()
 
-
-
-
-    def run(self, combat_system: CombatSystem, config=None, q_intermediate=None) -> bool:
         troops_a, troops_b = 0, 0
         for T in ['sea', 'land', 'air']:
             troops_a += self.army_a.units[T].sum() 
             troops_b += self.army_b.units[T].sum() 
 
         if troops_a == 0 or troops_b == 0:
-            return False
+            yield (False,100)
 
         if combat_system is CombatSystem.WarRoomV2 and config is None:
             logging.warning("NO CONFIG provided falling back to default config!!!!")
@@ -149,13 +204,6 @@ class Simulate:
         elif config is None:
             raise RuntimeError("No Config for combat system found")
 
-        #logging.basicConfig(level=logging.DEBUG)
-        #logging.debug(f"Army A Dice Ground {self.army_a.n_dice_ground}")
-        #logging.debug(f"Army B Dice Ground {self.army_b.n_dice_ground}")
-        #logging.debug(f"Army A Dice Air {self.army_a.n_dice_air}")
-        #logging.debug(f"Army B Dice Air {self.army_b.n_dice_air}")
-
-
         for n in tqdm(range(self.N)):
             self.cur_n = n
             battle = Battle(copy.copy(self.army_a), 
@@ -172,24 +220,72 @@ class Simulate:
                     if type == 'sea' and battle.army[side].submerged > 0:
                         self.survivors[side][type][-1][0] += battle.army[side].submerged
             if abrt:
-                break
+                yield (False, 100)
+
+            #if n % 50 == 0 and q_intermediate is not None and n != self.N and q_intermediate.empty():
+                #q_intermediate.put(self.intermediate_statistics())
+
+            yield (True, int(self.N / n))
+
+
+
+    def run(self, combat_system: CombatSystem, config=None, armyA: Optional[Army]=None, armyB: Optional[Army]=None) -> int:
+        
+        if armyA is not None:
+            self.army_a = armyA
+        if armyB is not None:
+            self.army_b = armyB
+
+        self.update_battle_type()
+
+        troops_a, troops_b = 0, 0
+        for T in ['sea', 'land', 'air']:
+            troops_a += self.army_a.units[T].sum() 
+            troops_b += self.army_b.units[T].sum() 
+
+        if troops_a == 0 or troops_b == 0:
+            return
+
+        if combat_system is CombatSystem.WarRoomV2 and config is None:
+            logging.warning("NO CONFIG provided falling back to default config!!!!")
+            config = wr20_vaniilla_options
+        elif config is None:
+            raise RuntimeError("No Config for combat system found")
+
+        for n in tqdm(range(self.N)):
+            self.cur_n = n
+            battle = Battle(copy.copy(self.army_a), 
+                            copy.copy(self.army_b),
+                            config)
+            status = battle.run(combat_system=combat_system)
+            #print(status)
+            abrt = self.running_stats(status)
+            self.statistics.append(status)
+
+            for side in ['A', 'B']:
+                for type in ['land', 'air', 'sea']:
+                    self.survivors[side][type].append(battle.army[side].units[type])
+                    if type == 'sea' and battle.army[side].submerged > 0:
+                        self.survivors[side][type][-1][0] += battle.army[side].submerged
+            if abrt:
+                return
+
+
 
-            if n % 50 == 0 and q_intermediate is not None:
-                q_intermediate.put(self.intermediate_statistics())
-        return True
 
     def intermediate_statistics(self):
-        self.stats[status] += 1
+
         N = (self.cur_n + 1)
         x = self.stats / N
+        x[np.isnan(x)] = 0
 
         report = StringIO(newline=os.linesep)
 
         report.write(f"Results:{os.linesep}")
-        report.write(f"{'A Won:':<20}{x[0] / N:.2f}{os.linesep}")
-        report.write(f"{'B Won:':<20}{x[1] / N:.2f}{os.linesep}") 
-        report.write(f"{'Draw:':<20}{x[2]  / N:.2f}{os.linesep}")
-        report.write(f"{'Mutual Annihilation:':<20}{x[3] / N:.2f}{os.linesep}")
+        report.write(f"{'A Won:':<20}{x[1]:.2f}{os.linesep}")
+        report.write(f"{'B Won:':<20}{x[0]:.2f}{os.linesep}") 
+        report.write(f"{'Draw:':<20}{x[2]:.2f}{os.linesep}")
+        report.write(f"{'Mutual Annihilation:':<20}{x[3]:.2f}{os.linesep}")
 
         return report.getvalue()
 
@@ -301,7 +397,7 @@ class Simulate:
                 # append air
                 str += f"{data['variations'][i][5+COLOR.GREEN]:<10}"
                 str += f"{data['variations'][i][5+COLOR.RED]:<10}"
-                buf.write(str+os.linesep
+                buf.write(str+os.linesep)
             return buf.getvalue()
 
         def print_legend() -> str:
@@ -350,4 +446,105 @@ class Simulate:
             report.write(print_avg_surv(self.metrics['avg_draw_b'], id='B - avg'))
             report.write(print_top_variations(self.metrics['outcomes_draw_b']))
 
-        return report.getvalue()
\ No newline at end of file
+        return report.getvalue()
+
+
+    def get_report_short(self, till=COLOR.BLACK) -> str:
+        report = StringIO(newline=os.linesep)
+
+        games_won_a     = self.metrics['games_won_a']     
+        games_won_b     = self.metrics['games_won_b']     
+        games_won_none  = self.metrics['games_won_none']  
+        games_draw      = self.metrics['games_draw']      
+
+        
+                                                          
+        def print_avg_surv(data, id='') -> str:
+            buf = StringIO()
+            str = f"{id:<10}"
+            for idx in range(till):
+                str += f"{data[idx]:<10.2f}"
+            # append air
+            str += f"{data[5+COLOR.GREEN]:<10.2f}"
+            str += f"{data[5+COLOR.RED]:<10.2f}"
+            buf.write(str+os.linesep) 
+            return buf.getvalue()
+        
+        def print_top_variations(data, N=3, p=1.0) -> str:
+            buf = StringIO(newline=os.linesep)
+            N = min(len(data['distribution']), N)
+
+            for i in range(N):
+                str = f"{data['distribution'][i]*p:<10.2f}"
+                for color in COLOR:
+                    if color == till:
+                        break
+                    str += f"{data['variations'][i][color.value]:<10}"
+                #buf.write(f"{data['distribution'][i]:<10.2f} {(data['variations'][i][0]):<10} {(data['variations'][i][1]):<10} {(data['variations'][i][2]):<10} {(data['variations'][i][3]):<10} {(data['variations'][i][4]):<10}{os.linesep}") 
+                # append air
+                str += f"{data['variations'][i][5+COLOR.GREEN]:<10}"
+                str += f"{data['variations'][i][5+COLOR.RED]:<10}"
+                buf.write(str+os.linesep)
+            return buf.getvalue()
+
+        def print_legend() -> str:
+            buf = StringIO()
+            str = f"{'':<10}"
+            for color in COLOR:
+                if color == till:
+                    break
+                str += f"{color.name:<10}"
+            # append air
+            str += f"{'AIR Green':<10}"
+            str += f"{'AIR Red':<10}"
+            buf.write(str+os.linesep)
+            return buf.getvalue()
+        N = self.cur_n+1
+        report.write("--------------------------------------------------------------"+os.linesep)
+        report.write(f"Results:{os.linesep}")
+        report.write(f"{'A Won:':<20}{games_won_a / N:.2f}{os.linesep}")
+        report.write(f"{'B Won:':<20}{games_won_b / N:.2f}{os.linesep}") 
+        report.write(f"{'Draw:':<20}{games_draw /   N:.2f}{os.linesep}")
+        report.write(f"{'Mutual Annihilation:':<20}{games_won_none / N:.2f}{os.linesep}")
+
+        report.write("--------------------------------------------------------------"+os.linesep)
+        report.write("Total Average"+os.linesep)
+        report.write(print_legend())
+        report.write(print_avg_surv(self.metrics['avg_surv_a'], id='A-AVG'))
+        report.write(print_avg_surv(self.metrics['avg_surv_b'], id='B-AVG'))
+        report.write("--------------------------------------------------------------"+os.linesep)
+
+
+        if games_won_a > 0:
+            p_a = games_won_a/N
+            report.write(os.linesep)
+            report.write(f'Army A Won - {p_a:.2f}{os.linesep}')
+            report.write(print_legend())
+            report.write(print_avg_surv(self.metrics['avg_surv_a'], id='AVG'))
+            report.write("--------------------------------------------------------------"+os.linesep)
+            report.write(print_top_variations(self.metrics['outcomes_won_a'], p=p_a))
+
+
+        if games_won_b > 0:
+            p_b = games_won_b/N
+            report.write(os.linesep)
+            report.write(f'Army B Won - {p_b:.2f}{os.linesep}')
+            report.write(print_legend())
+            report.write(print_avg_surv(self.metrics['avg_surv_b'], id='AVG'))
+            report.write("--------------------------------------------------------------"+os.linesep)
+            report.write(print_top_variations(self.metrics['outcomes_won_b'], p=p_b))
+
+        if games_draw > 0:
+            g_d = games_draw/N
+            report.write(os.linesep)
+            report.write(f'Draw - {g_d:.2f}{os.linesep}')
+            report.write(print_legend())
+            report.write("--------------------------------------------------------------"+os.linesep)
+            report.write(print_avg_surv(self.metrics['avg_draw_a'], id='A - avg'))
+            report.write(print_top_variations(self.metrics['outcomes_draw_a'], p=g_d))
+            report.write("--------------------------------------------------------------"+os.linesep)
+            report.write(print_avg_surv(self.metrics['avg_draw_b'], id='B - avg'))
+            report.write(print_top_variations(self.metrics['outcomes_draw_b'], p=g_d))
+
+        return report.getvalue()
+
